//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop

#include "HashCashChks.h"
#include "HashCashDlgs.h"
#include "MsgBoxDlgs.h"
#include "global.h"
//---------------------------------------------------------------------------
bool __fastcall THashCashDlg::ValidInput()
{
   if (!ValidBits()) return false;
   if (!ValidResource()) return false;
   if (TimeBox->Checked) return true;
   if (!ValidTokens()) return false;
   if (!ValidValidity()) return false;
   if (!ValidRandom()) return false;
   return true;
}
//---------------------------------------------------------------------------
AnsiString __fastcall InvalidChars(AnsiString inString, AnsiString legalchars)
{
   AnsiString badchars;
   for (int i=1; i<=inString.Length(); i++)
   {
      AnsiString achar = inString.SubString(i,1);
      if (legalchars.Pos(achar)==0)
         badchars += achar;
   }
   return badchars;
}
//---------------------------------------------------------------------------
bool __fastcall THashCashDlg::ValidBits()
{
   AnsiString badchars;
   if ((badchars=InvalidChars(BitsEdit->Text,"0123456789"))!="")
   {
      MsgDlg(this,
         "Your 'Bits' field contains the invalid character(s) '"+badchars+"'.",
         "Bits Field Error",MB_OK);
      return false;
   }
   if (BitsEdit->Text.ToIntDef(0)<1)
   {
      MsgDlg(this,
         "The value for your 'Bits' field must be 1 or greater.",
         "Bits Field Error",MB_OK);
      return false;
   }
   return true;
}
//---------------------------------------------------------------------------
bool __fastcall THashCashDlg::ValidResource()
{
   if (Resource->Text.IsEmpty())
   {
      MsgDlg(this,
         "You must name a 'resource' to mint a token.",
         "Resource Field Error",MB_OK);
      return false;
   }
   if (Resource->Text.Pos(" ")>0)
   {
      MsgDlg(this,
         "The resource name you have specify cannot contain space characters.",
         "Resource Field Error",MB_OK);
      return false;
   }
   return true;
}
//---------------------------------------------------------------------------
bool __fastcall THashCashDlg::ValidTokens()
{
   AnsiString badchars;
   if ((badchars=InvalidChars(QtyEdit->Text,"0123456789"))!="")
   {
      MsgDlg(this,
         "Your 'Tokens' field contains the invalid character(s) '"+badchars+"'.",
         "Tokens Field Error",MB_OK);
      return false;
   }
   if (QtyEdit->Text.ToIntDef(0)<1)
   {
      MsgDlg(this,
         "You must mint at least one token!",
         "Tokens Field Error",MB_OK);
      return false;
   }
   return true;
}
//---------------------------------------------------------------------------
bool __fastcall THashCashDlg::ValidValidity()
{
   int pos,val;
   AnsiString badchars,expire=ExpireTime->Text;
   if ((badchars=InvalidChars(expire,"0123456789:s"))!="")
   {
      MsgDlg(this,
         "Your 'Validity' field contains the invalid character(s) '"+badchars+"'.",
         "Options, Validity Field Error",MB_OK);
      return false;
   }
   if (expire.Pos("s")>0 && expire.Pos(":")>0)
   {
      MsgDlg(this,
         s("Your 'Validity' field contains both 's' and ':'. These characters\n")+
         s("are mutually exclusive. See help topic 'Options, Validity Period'"),
         s("Options, Validity Field Error"),MB_OK);
      return false;
   }
   if (expire.Pos("s")>0 && expire.Pos("s")<expire.Length())
   {
      MsgDlg(this,
         s("Your 'Validity' field contains 's' but it is not the last character\n")+
         s("in the field. Move the 's' to the end."),
         s("Options, Validity Field Error"),MB_OK);
      return false;
   }
   if (expire.Pos(":")==1)
   {
      MsgDlg(this,
         s("Your 'Validity' field has a ':' character in the first position.\n")+
         s("This must be proceeded by at least 1 digit."),
         s("Options, Validity Field Error"),MB_OK);
      return false;
   }
   if (expire.Pos("::")>1)
   {
      MsgDlg(this,
         s("Your 'Validity' field has two '::' characters side by side.\n")+
         s("These must be separated by at least 1 digit."),
         s("Options, Validity Field Error"),MB_OK);
      return false;
   }
   if ((pos=expire.Pos("s"))>0 && GetToken(0,expire,"s").ToIntDef(0)<0)
   {
      MsgDlg(this,
         "Your 'Validity Period' field must be 0 or greater.",
         "Options, Validity Field Error",MB_OK);
      return false;
   }
   if (pos>0) return true;

   if (GetToken(0,expire,":").ToIntDef(0)<0)
   {
      MsgDlg(this,
         "The value for 'Validity Period, Days' must be 0 or greater.",
         "Options, Validity Field Error",MB_OK);
      return false;
   }
   if ((val=GetToken(1,expire,":").ToIntDef(0))<0 || val>23)
   {
      MsgDlg(this,
         "The value for 'Validity Period, Hours' must be between 0 and 23.",
         "Options, Validity Field Error",MB_OK);
      return false;
   }
   if ((val=GetToken(2,expire,":").ToIntDef(0))<0 || val>59)
   {
      MsgDlg(this,
         "The value for 'Validity Period, Minutes' must be between 0 and 59.",
         "Options, Validity Field Error",MB_OK);
      return false;
   }
   if ((val=GetToken(3,expire,":").ToIntDef(0))<0 || val>59)
   {
      MsgDlg(this,
         "The value for 'Validity Period, Seconds' must be between 0 and 59.",
         "Options, Validity Field Error",MB_OK);
      return false;
   }
   return true;
}
//---------------------------------------------------------------------------
bool __fastcall THashCashDlg::ValidRandom()
{
   int val;
   AnsiString badchars,anon=AnonTime->Text;
   if ((badchars=InvalidChars(anon,"-0123456789:s"))!="")
   {
      MsgDlg(this,
         "Your 'Rand Period' field contains the invalid character(s) '"+badchars+"'.",
         "Options, Rand Field Error",MB_OK);
      return false;
   }
   if (anon.Pos("s")>0 && anon.Pos(":")>0)
   {
      MsgDlg(this,
         s("Your 'Rand Period' field contains both 's' and ':'. These characters\n")+
         s("are mutually exclusive. See help topic 'Options, Rand Period'"),
         s("Options, Rand Field Error"),MB_OK);
      return false;
   }
   if ((val=anon.Pos("s"))>0 && val<anon.Length())
   {
      MsgDlg(this,
         s("Your 'Rand Period' field contains 's' but it is not the last character\n")+
         s("in the field. Move the 's' to the end."),
         s("Options, Rand Field Error"),MB_OK);
      return false;
   }
   if (anon.Pos(":")==1)
   {
      MsgDlg(this,
         s("Your 'Rand Period' field has a ':' character in the first position.\n")+
         s("This must be proceeded by at least 1 digit."),
         s("Options, Rand Field Error"),MB_OK);
      return false;
   }
   if (anon.Pos("::")>1)
   {
      MsgDlg(this,
         s("Your 'Rand Period' field has two ':' characters side by side.\n")+
         s("These must be separated by at least 1 digit."),
         "Options, Rand Field Error",MB_OK);
      return false;
   }
   if (anon.Pos("s")>0) return true;

   if ((val=GetToken(1,anon,":").ToIntDef(0))<0 || val>23)
   {
      MsgDlg(this,
         "The value for 'Rand Period, Hours' must be between 0 and 23.",
         "Options, Rand Field Error",MB_OK);
      return false;
   }
   if ((val=GetToken(2,anon,":").ToIntDef(0))<0 || val>59)
   {
      MsgDlg(this,
         "The value for 'Rand Period, Minutes' must be between 0 and 59.",
         "Options, Rand Field Error",MB_OK);
      return false;
   }
   if ((val=GetToken(3,anon,":").ToIntDef(0))<0 || val>59)
   {
      MsgDlg(this,
         "The value for 'Rand Period, Seconds' must be between 0 and 59.",
         "Options, Rand Field Error",MB_OK);
      return false;
   }
   return true;
}
//---------------------------------------------------------------------------
